@@ -1,6 +1,6 @@
 /*
     DeaDBeeF -- the music player
-    Copyright (C) 2009-2014 Alexey Yakovenko and other contributors
+    Copyright (C) 2009-2014 Oleksiy Yakovenko and other contributors
 
     This software is provided 'as-is', without any express or implied
     warranty.  In no event will the authors be held liable for any damages
@@ -21,33 +21,39 @@
     3. This notice may not be removed or altered from any source distribution.
 */
 
-#import "AppDelegate.h"
-#import "dispatch/dispatch.h"
-#import "DdbWidgetManager.h"
-#import "ReplayGainScannerController.h"
-#import "DdbShared.h"
-#import "NowPlayable.h"
-#import "LogWindowController.h"
-#import "HelpWindowController.h"
-#import "EqualizerWindowController.h"
-#import "NSMenu+ActionItems.h"
-#include "conf.h"
-#include "streamer.h"
-#include "junklib.h"
-#include "../../common.h"
-#include "../../deadbeef.h"
-#include "../../plugins/artwork/artwork.h"
 #include <sys/time.h>
+#include <deadbeef/common.h>
+#include <deadbeef/deadbeef.h>
+#include "../../plugins/artwork/artwork.h"
+#import "AppDelegate.h"
+#import "conf.h"
+#import "CoverManager.h"
+#import "DdbShared.h"
+#import "DdbUndoBufferRetainer.h"
+#import "DesignModeDefs.h"
+#import "DesignModeState.h"
+#import "EqualizerWindowController.h"
+#import "HelpWindowController.h"
+#import "junklib.h"
+#import "KeyboardShortcuts/KeyboardShortcutManager.h"
+#import "KeyboardShortcuts/KeyboardShortcutManager+ViewController.h"
+#import "KeyboardShortcuts/KeyboardShortcutEditorViewController.h"
+#import "KeyboardShortcuts/KeyboardShortcutEditorWindowController.h"
+#import "LogWindowController.h"
+#import "NowPlayable.h"
+#import "NSMenu+ActionItems.h"
+#import "ReplayGainScannerController.h"
+#import "TrackPropertiesManager.h"
+#import "streamer.h"
+#import "UndoIntegration.h"
 
 extern DB_functions_t *deadbeef;
 
-extern BOOL g_CanQuit;
-
-AppDelegate *g_appDelegate;
-
 @interface AppDelegate () {
     char *_titleScript;
     char *_artistAlbumScript;
+    int _listenerFileAddBeginAddIdentifier;
+    int _listenerFileAddedIdentifier;
 }
 
 
@@ -70,12 +76,27 @@ AppDelegate *g_appDelegate;
 
 @property (nonatomic,readwrite) MediaLibraryManager *mediaLibraryManager;
 
-@property (nonatomic) DesignableViewController *rootViewController;
+@property (weak) IBOutlet NSMenuItem *designModeMenuItem;
+@property DesignModeState *designModeState;
 
+@property (nonatomic) KeyboardShortcutEditorWindowController *keyboardShortcutEditorWindowController;
+@property (nonatomic) KeyboardShortcutEditorViewController *keyboardShortcutEditorViewController;
 
 @end
 
 @implementation AppDelegate
+- (instancetype)init {
+    self = [super init];
+    if (self == nil) {
+        return nil;
+    }
+
+    // initialize design mode, to avoid random background thread calls
+    id<DesignModeStateProtocol> state = DesignModeState.sharedInstance;
+    [state load];
+
+    return self;
+}
 
 - (void)volumeChanged {
     [_mainWindow updateVolumeBar];
@@ -95,27 +116,27 @@ AppDelegate *g_appDelegate;
         _orderShuffleAlbums,
         nil
     };
-    
+
     ddb_shuffle_t shuffle = deadbeef->streamer_get_shuffle ();
     for (ddb_shuffle_t i = 0; shuffle_items[i]; i++) {
         shuffle_items[i].state = i==shuffle?NSControlStateValueOn:NSControlStateValueOff;
     }
-    
+
     NSMenuItem *repeat_items[] = {
         _loopAll,
         _loopNone,
         _loopSingle,
         nil
     };
-    
+
     ddb_repeat_t repeat = deadbeef->streamer_get_repeat ();
     for (ddb_repeat_t i = 0; repeat_items[i]; i++) {
         repeat_items[i].state = i==repeat?NSControlStateValueOn:NSControlStateValueOff;
     }
-    
+
     _scrollFollowsPlayback.state = deadbeef->conf_get_int ("playlist.scroll.followplayback", 1)?NSControlStateValueOn:NSControlStateValueOff;
     _cursorFollowsPlayback.state = deadbeef->conf_get_int ("playlist.scroll.cursorfollowplayback", 1)?NSControlStateValueOn:NSControlStateValueOff;
-    
+
     _stopAfterCurrent.state = deadbeef->conf_get_int ("playlist.stop_after_current", 0)?NSControlStateValueOn:NSControlStateValueOff;
     _stopAfterCurrentAlbum.state = deadbeef->conf_get_int ("playlist.stop_after_album", 0)?NSControlStateValueOn:NSControlStateValueOff;
 
@@ -130,33 +151,35 @@ AppDelegate *g_appDelegate;
 static int fileadd_cancelled = 0;
 
 static void fileadd_begin (ddb_fileadd_data_t *data, void *user_data) {
+    AppDelegate *appDelegate = (__bridge AppDelegate *)user_data;
     fileadd_cancelled = 0;
     dispatch_async(dispatch_get_main_queue(), ^{
-        [g_appDelegate.mainWindow.window beginSheet:g_appDelegate.addFilesWindow completionHandler:^(NSModalResponse returnCode) {
-
+        [appDelegate.mainWindow.window beginSheet:appDelegate.addFilesWindow completionHandler:^(NSModalResponse returnCode) {
         }];
     });
 }
 
 static void fileadd_end (ddb_fileadd_data_t *data, void *user_data) {
+    AppDelegate *appDelegate = (__bridge AppDelegate *)user_data;
     dispatch_async(dispatch_get_main_queue(), ^{
-        [g_appDelegate.mainWindow.window endSheet:g_appDelegate.addFilesWindow returnCode:NSModalResponseOK];
-        [[[g_appDelegate mainWindow] window] makeKeyAndOrderFront:g_appDelegate];
+        [appDelegate.mainWindow.window endSheet:appDelegate.addFilesWindow returnCode:NSModalResponseOK];
+        [appDelegate.mainWindow.window makeKeyAndOrderFront:appDelegate];
     });
 }
 
 static BOOL _settingLabel = NO;
 
 static int file_added (ddb_fileadd_data_t *data, void *user_data) {
+    AppDelegate *appDelegate = (__bridge AppDelegate *)user_data;
     const char *uri = deadbeef->pl_find_meta (data->track, ":URI");
     if (!_settingLabel) {
         // HACK: we want to set the label asynchronously, to minimize delays,
-        // but we also want to avoid sending multiple labels, becuase that's meaningless.
+        // but we also want to avoid sending multiple labels, because that's meaningless.
         // So we use a basic flag, to see if the label is being set already.
-        NSString *s = [NSString stringWithUTF8String:uri];
+        NSString *s = @(uri);
         _settingLabel = YES;
         dispatch_async(dispatch_get_main_queue(), ^{
-            g_appDelegate.addFilesLabel.stringValue = s;
+            appDelegate.addFilesLabel.stringValue = s;
             _settingLabel = NO;
         });
     }
@@ -168,8 +191,10 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
 }
 
 - (void)awakeFromNib {
+    self.designModeState = DesignModeState.sharedInstance;
     self.mediaLibraryManager = [MediaLibraryManager new];
     [self initMainMenu];
+    [KeyboardShortcutManager.shared updateWithMenu:self.mainMenu];
     [self initMainWindow];
     [self initSearchWindow];
     [self initLogWindow];
@@ -180,7 +205,8 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
 }
 
 - (void)initMainWindow {
-    _mainWindow = [[MainWindowController alloc] initWithWindowNibName:@"MainWindow"];
+    _mainWindow = [[MainWindowController alloc] initWithWindowNibName:@"SplitViewMainWindow"];
+    _mainWindow.mediaLibraryManager = self.mediaLibraryManager;
     _mainWindow.shouldCascadeWindows = NO;
     _mainWindow.window.releasedWhenClosed = NO;
     _mainWindow.window.excludedFromWindowsMenu = YES;
@@ -198,10 +224,9 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
 }
 
 - (void)initMainMenu {
-    // add new actions
-    [self.mainMenu addActionItemsForContext:DDB_ACTION_CTX_MAIN track:nil filter:^BOOL(DB_plugin_action_t * _Nonnull action) {
-        return (action->flags & (DB_ACTION_COMMON|DB_ACTION_ADD_MENU)) == (DB_ACTION_COMMON|DB_ACTION_ADD_MENU);
-    }];
+    [self.mainMenu addPluginActionItemsForSelectedTrack:NULL selectedCount:0 actionContext:DDB_ACTION_CTX_MAIN];
+
+    self.designModeMenuItem.state = self.designModeState.enabled ? NSControlStateValueOn : NSControlStateValueOff;
 }
 
 - (BOOL)equalizerAvailable {
@@ -223,10 +248,6 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
 }
 
 - (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {
-    if (g_CanQuit) {
-        return NSTerminateNow;
-    }
-
     if (deadbeef->have_background_jobs ()) {
         NSAlert *alert = [NSAlert new];
         alert.messageText = @"Do you really want to quit now?";
@@ -239,6 +260,7 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
         }
     }
 
+    CoverManager.shared.isTerminating = YES;
     deadbeef->sendmessage(DB_EV_TERMINATE, 0, 0, 0);
     return NSTerminateLater;
 }
@@ -248,6 +270,7 @@ main_cleanup_and_quit (void);
 
 - (void)applicationWillTerminate:(NSNotification *)notification {
     @autoreleasepool {
+        [DdbUndoBufferRetainer cleanupShared];
         [ConverterWindowController cleanup];
         [ReplayGainScannerController cleanup];
         [_searchWindow close];
@@ -260,9 +283,21 @@ main_cleanup_and_quit (void);
         [_mainWindow cleanup];
         [self.mainWindow.window close];
         self.mainWindow = nil;
+        [TrackPropertiesManager deinitializeSharedInstance];
+        self.mediaLibraryManager = nil;
+
+        self.designModeState = nil;
+        [DesignModeState freeSharedInstance];
+
+        deadbeef->unlisten_file_add_beginend(_listenerFileAddBeginAddIdentifier);
+        deadbeef->unlisten_file_added(_listenerFileAddedIdentifier);
     }
-    self.mediaLibraryManager = nil;
     main_cleanup_and_quit();
+    // main_cleanup_and_quit will call "exit" after async jobs finish, which may occur on another thread.
+    // Therefore inifinite wait here.
+    for (;;) {
+        usleep(10000000);
+    }
 }
 
 - (void)bind {
@@ -281,8 +316,7 @@ main_cleanup_and_quit (void);
     [_equalizerMenuItem unbind:@"hidden"];
 }
 
-- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
-{
+- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
     // high sierra would terminate the app on SIGPIPE by default, which breaks converter error handling
     signal(SIGPIPE, SIG_IGN);
 
@@ -290,19 +324,25 @@ main_cleanup_and_quit (void);
 
     // initialize gui from settings
     [self configChanged];
-    
-    deadbeef->listen_file_add_beginend (fileadd_begin, fileadd_end, NULL);
-    deadbeef->listen_file_added (file_added, NULL);
 
-    g_appDelegate = self;
+    _listenerFileAddBeginAddIdentifier = deadbeef->listen_file_add_beginend (fileadd_begin, fileadd_end, (__bridge void *)(self));
+    _listenerFileAddedIdentifier = deadbeef->listen_file_added (file_added, (__bridge void *)(self));
 
 #if !DISABLE_MM_KEY_GRABBER
     self.nowPlayable = [NowPlayable new];
 #endif
 
     [self updateDockNowPlaying];
+
+    id<DesignModeStateProtocol> state = DesignModeState.sharedInstance;
+    [state.rootWidget configure];
+
+    [self.mainWindow.window recalculateKeyViewLoop];
+    [self.mainWindow setupInitialFirstResponder:state.rootWidget];
 }
 
+
+
 - (BOOL)applicationShouldHandleReopen:(NSApplication *)sender hasVisibleWindows:(BOOL)flag{
     _mainWindow.window.isVisible = YES;
     return YES;
@@ -312,7 +352,7 @@ main_cleanup_and_quit (void);
     dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
     dispatch_async(aQueue, ^{
         char str[100];
-        add_paths([filename UTF8String], (int)[filename length], 0, str, 100);
+        add_paths(filename.UTF8String, (int)filename.length, 0, str, 100);
     });
     return YES; // assume that everything went ok
 }
@@ -322,15 +362,16 @@ main_cleanup_and_quit (void);
     dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
     dispatch_async(aQueue, ^{
         char str[100];
+        NSArray *sortedFilenames = [filenames sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
         // building single paths string for the deadbeef function, paths must be separated by '\0'
-        NSString *paths =[filenames componentsJoinedByString:@"\0"];
-        add_paths([paths UTF8String], (int)[paths length], 0, str, 100);
+        NSString *paths =[sortedFilenames componentsJoinedByString:@"\0"];
+        add_paths(paths.UTF8String, (int)[paths lengthOfBytesUsingEncoding:NSUTF8StringEncoding], 0, str, 100);
     });
 }
 
 
 - (IBAction)showMainWinAction:(id)sender {
-    BOOL vis = ![_mainWindow.window isVisible];
+    BOOL vis = !(_mainWindow.window).visible;
     _mainWindow.window.isVisible = vis;
     if (vis) {
         [_mainWindow.window makeKeyWindow];
@@ -338,7 +379,7 @@ main_cleanup_and_quit (void);
 }
 
 - (IBAction)showLogWindowAction:(id)sender {
-    BOOL vis = ![_logWindow.window isVisible];
+    BOOL vis = !(_logWindow.window).visible;
     _logWindow.window.isVisible = vis;
     if (vis) {
         [_logWindow.window makeKeyWindow];
@@ -346,7 +387,7 @@ main_cleanup_and_quit (void);
 }
 
 - (IBAction)showEqualizerWindowAction:(id)sender {
-    BOOL vis = ![_equalizerWindow.window isVisible];
+    BOOL vis = !(_equalizerWindow.window).visible;
     _equalizerWindow.window.isVisible = vis;
     if (vis) {
         [_equalizerWindow.window makeKeyWindow];
@@ -401,32 +442,52 @@ main_cleanup_and_quit (void);
     openDlg.canChooseDirectories = NO;
     if ( [openDlg runModal] == NSModalResponseOK )
     {
-        NSArray* files = [openDlg URLs];
-        ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-        if (!deadbeef->plt_add_files_begin (plt, 0)) {
-            if (clear) {
-                deadbeef->plt_clear(plt);
-                deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
-            }
-            dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
-            dispatch_async(aQueue, ^{
+        NSArray* files = openDlg.URLs;
+        ddb_playlist_t *plt = deadbeef->plt_alloc("open-files");
+        ddb_playlist_t *plt_curr = deadbeef->plt_get_curr ();
+        if (clear) {
+            deadbeef->plt_clear(plt_curr);
+            deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
+        }
+        if (!deadbeef->plt_add_files_begin (plt_curr, 0)) {
+            dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
                 for (NSUInteger i = 0; i < files.count; i++) {
-                    NSString* fileName = [[files objectAtIndex:i] path];
+                    NSString* fileName = [files[i] path];
                     if (fileName) {
-                        deadbeef->plt_add_file2 (0, plt, [fileName UTF8String], NULL, NULL);
+                        deadbeef->plt_add_file2 (0, plt, fileName.UTF8String, NULL, NULL);
+                        if (fileadd_cancelled) {
+                            break;
+                        }
                     }
                 }
-                deadbeef->plt_add_files_end (plt, 0);
-                deadbeef->plt_unref (plt);
-                deadbeef->pl_save_current();
-                if (play) {
-                    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
-                    deadbeef->sendmessage (DB_EV_PLAY_NUM, 0, 0, 0);
+                if (!fileadd_cancelled) {
+                    dispatch_async(dispatch_get_main_queue(), ^{
+                        ddb_playItem_t *tail = deadbeef->plt_get_tail_item(plt_curr, PL_MAIN);
+                        ddb_undo->set_action_name ("Add Files");
+                        deadbeef->plt_move_all_items(plt_curr, plt, tail);
+                        if (tail != NULL) {
+                            deadbeef->pl_item_unref (tail);
+                        }
+                        deadbeef->pl_save_current();
+                        deadbeef->plt_add_files_end (plt_curr, 0);
+                        deadbeef->plt_unref (plt);
+                        deadbeef->plt_unref (plt_curr);
+
+                        if (play) {
+                            deadbeef->sendmessage (DB_EV_PLAY_NUM, 0, 0, 0);
+                        }
+                    });
+                }
+                else {
+                    deadbeef->plt_add_files_end (plt_curr, 0);
+                    deadbeef->plt_unref (plt);
+                    deadbeef->plt_unref (plt_curr);
                 }
             });
         }
         else {
             deadbeef->plt_unref (plt);
+            deadbeef->plt_unref (plt_curr);
         }
     }
 }
@@ -446,25 +507,41 @@ main_cleanup_and_quit (void);
     openDlg.canChooseDirectories = YES;
     if ( [openDlg runModal] == NSModalResponseOK )
     {
-        NSArray* files = [openDlg URLs];
-        ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-        if (!deadbeef->plt_add_files_begin (plt, 0)) {
-            dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
-            dispatch_async(aQueue, ^{
+        NSArray* files = openDlg.URLs;
+        ddb_playlist_t *plt = deadbeef->plt_alloc ("add-folders");
+        ddb_playlist_t *plt_curr = deadbeef->plt_get_curr ();
+        if (!deadbeef->plt_add_files_begin (plt_curr, 0)) {
+            dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
                 for (NSUInteger i = 0; i < files.count; i++) {
-                    NSString *fileName = [[files objectAtIndex:i] path];
+                    NSString *fileName = [files[i] path];
                     if (fileName) {
-                        deadbeef->plt_add_dir2 (0, plt, [fileName UTF8String], NULL, NULL);
+                        deadbeef->plt_add_dir2 (0, plt, fileName.UTF8String, NULL, NULL);
                     }
                 }
-                deadbeef->plt_add_files_end (plt, 0);
-                deadbeef->plt_unref (plt);
-                deadbeef->pl_save_current();
-                deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
+                if (!fileadd_cancelled) {
+                    dispatch_async(dispatch_get_main_queue(), ^{
+                        ddb_playItem_t *tail = deadbeef->plt_get_tail_item(plt_curr, PL_MAIN);
+                        deadbeef->plt_move_all_items(plt_curr, plt, tail);
+                        if (tail != NULL) {
+                            deadbeef->pl_item_unref (tail);
+                        }
+                        deadbeef->pl_save_current();
+                        ddb_undo->set_action_name ("Add Folders");
+                        deadbeef->plt_add_files_end (plt_curr, 0);
+                        deadbeef->plt_unref (plt);
+                        deadbeef->plt_unref (plt_curr);
+                    });
+                }
+                else {
+                    deadbeef->plt_add_files_end (plt_curr, 0);
+                    deadbeef->plt_unref (plt);
+                    deadbeef->plt_unref (plt_curr);
+                }
             });
         }
         else {
             deadbeef->plt_unref (plt);
+            deadbeef->plt_unref (plt_curr);
         }
     }
 }
@@ -473,24 +550,32 @@ main_cleanup_and_quit (void);
     _addLocationTextField.stringValue = @"";
     [_mainWindow.window beginSheet:_addLocationPanel completionHandler:^(NSModalResponse returnCode) {
         if (returnCode == NSModalResponseOK) {
-            NSString *text = [self.addLocationTextField stringValue];
+            NSString *text = [self.addLocationTextField.stringValue stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
 
-            ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-            if (!deadbeef->plt_add_files_begin (plt, 0)) {
+            ddb_playlist_t *plt = deadbeef->plt_alloc ("add-location");
+            ddb_playlist_t *plt_curr = deadbeef->plt_get_curr ();
+            if (!deadbeef->plt_add_files_begin (plt_curr, 0)) {
                 dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
                 dispatch_async(aQueue, ^{
                     DB_playItem_t *tail = deadbeef->plt_get_last (plt, PL_MAIN);
-                    deadbeef->plt_insert_file2 (0, plt, tail, [text UTF8String], NULL, NULL, NULL);
+                     deadbeef->plt_insert_file2 (0, plt, tail, text.UTF8String, NULL, NULL, NULL);
+                    deadbeef->plt_move_all_items(plt_curr, plt, tail);
                     if (tail) {
                         deadbeef->pl_item_unref (tail);
                     }
-                    deadbeef->plt_add_files_end (plt, 0);
-                    deadbeef->plt_unref (plt);
                     deadbeef->pl_save_current ();
+
+                    dispatch_async(dispatch_get_main_queue(), ^{
+                        ddb_undo->set_action_name ("Add Location");
+                        deadbeef->plt_add_files_end (plt_curr, 0);
+                        deadbeef->plt_unref (plt);
+                        deadbeef->plt_unref (plt_curr);
+                    });
                 });
             }
             else {
                 deadbeef->plt_unref (plt);
+                deadbeef->plt_unref (plt_curr);
             }
         }
     }];
@@ -548,14 +633,14 @@ main_cleanup_and_quit (void);
 
 - (IBAction)sortPlaylistCustom:(id)sender {
     deadbeef->pl_lock ();
-    _customSortEntry.stringValue = [NSString stringWithUTF8String:deadbeef->conf_get_str_fast ("cocoaui.custom_sort_tf", "")];
+    _customSortEntry.stringValue = @(deadbeef->conf_get_str_fast ("cocoaui.custom_sort_tf", ""));
     deadbeef->pl_unlock ();
     _customSortDescending.state = deadbeef->conf_get_int ("cocoaui.sort_desc", 0) ? NSControlStateValueOn : NSControlStateValueOff;
     [_mainWindow.window beginSheet:_customSortPanel completionHandler:^(NSModalResponse returnCode) {
         NSInteger state = self.customSortDescending.state;
         self.descendingSortMode.state =  state;
         deadbeef->conf_set_int ("cocoaui.sort_desc", state == NSControlStateValueOn ? 1 : 0);
-        deadbeef->conf_set_str ("cocoaui.custom_sort_tf", [[self.customSortEntry stringValue] UTF8String]);
+        deadbeef->conf_set_str ("cocoaui.custom_sort_tf", (self.customSortEntry).stringValue.UTF8String);
 
         if (returnCode == NSModalResponseOK) {
             [self sortPlaylistByTF:self.customSortEntry.stringValue.UTF8String];
@@ -580,12 +665,6 @@ main_cleanup_and_quit (void);
     deadbeef->conf_save ();
 }
 
-- (IBAction)delete:(id)sender {
-    deadbeef->pl_delete_selected ();
-    deadbeef->pl_save_current();
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
-}
-
 - (IBAction)orderLinearAction:(id)sender {
     deadbeef->streamer_set_shuffle (DDB_SHUFFLE_OFF);
 }
@@ -638,6 +717,10 @@ main_cleanup_and_quit (void);
     deadbeef->sendmessage(DB_EV_NEXT, 0, 0, 0);
 }
 
+- (IBAction)playRandomAction:(id)sender {
+    deadbeef->sendmessage(DB_EV_PLAY_RANDOM, 0, 0, 0);
+}
+
 - (IBAction)cursorFollowsPlaybackAction:(id)sender {
 }
 
@@ -658,72 +741,41 @@ main_cleanup_and_quit (void);
     deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
 }
 
-- (void)selectAll:(id)sender {
-    deadbeef->pl_select_all ();
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_SELECTION, 0);
-}
-
-- (IBAction)deselectAllAction:(id)sender {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->pl_get_first (PL_MAIN);
-    while (it) {
-        if (deadbeef->pl_is_selected (it)) {
-            deadbeef->pl_set_selected (it, 0);
-        }
-        DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-        deadbeef->pl_item_unref (it);
-        it = next;
+- (int)ddb_message:(int)_id ctx:(uint64_t)ctx p1:(uint32_t)p1 p2:(uint32_t)p2 {
+    if (_id == DB_EV_PLAYLISTCHANGED) {
+        dispatch_async(dispatch_get_main_queue(), ^{
+            deadbeef->undo_process();
+        });
     }
-    deadbeef->pl_unlock ();
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_SELECTION, 0);
-}
 
-- (IBAction)invertSelectionAction:(id)sender {
-    deadbeef->pl_lock ();
-    DB_playItem_t *it = deadbeef->pl_get_first (PL_MAIN);
-    while (it) {
-        if (deadbeef->pl_is_selected (it)) {
-            deadbeef->pl_set_selected (it, 0);
-        }
-        else {
-            deadbeef->pl_set_selected (it, 1);
-        }
-        DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-        deadbeef->pl_item_unref (it);
-        it = next;
-    }
-    deadbeef->pl_unlock ();
-    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_SELECTION, 0);
-}
+    [self.mainWindow.sidebarOutlineViewController.mediaLibraryOutlineViewController  widgetMessage:_id ctx:ctx p1:p1 p2:p2];
 
-- (IBAction)selectionCropAction:(id)sender {
-}
-
-+ (int)ddb_message:(int)_id ctx:(uint64_t)ctx p1:(uint32_t)p1 p2:(uint32_t)p2
-{
-    [[DdbWidgetManager defaultWidgetManager] widgetMessage:_id ctx:ctx p1:p1 p2:p2];
-
-    [g_appDelegate.mainWindow.rootViewController sendMessage:_id ctx:ctx p1:p1 p2:p2];
-    [(DesignableViewController *)(g_appDelegate.searchWindow.viewController) sendMessage:_id ctx:ctx p1:p1 p2:p2];
+    [DesignModeState.sharedInstance.rootWidget message:_id ctx:ctx p1:p1 p2:p2];
+    [self.searchWindow.viewController sendMessage:_id ctx:ctx p1:p1 p2:p2];
 
     if (_id == DB_EV_CONFIGCHANGED) {
-        [g_appDelegate performSelectorOnMainThread:@selector(configChanged) withObject:nil waitUntilDone:NO];
+        [self performSelectorOnMainThread:@selector(configChanged) withObject:nil waitUntilDone:NO];
     }
     else if (_id == DB_EV_SONGSTARTED) {
-        [g_appDelegate performSelectorOnMainThread:@selector(updateDockNowPlaying) withObject:nil waitUntilDone:NO];
+        [self performSelectorOnMainThread:@selector(updateDockNowPlaying) withObject:nil waitUntilDone:NO];
     }
     else if (_id == DB_EV_SONGFINISHED) {
-        [g_appDelegate performSelectorOnMainThread:@selector(clearDockNowPlaying) withObject:nil waitUntilDone:NO];
+        [self performSelectorOnMainThread:@selector(clearDockNowPlaying) withObject:nil waitUntilDone:NO];
     }
     else if (_id == DB_EV_SONGCHANGED || _id == DB_EV_TRACKINFOCHANGED || (_id == DB_EV_PLAYLISTCHANGED && p1 == DDB_PLAYLIST_CHANGE_CONTENT)) {
-        [g_appDelegate performSelectorOnMainThread:@selector(updateTitleBar) withObject:nil waitUntilDone:NO];
+        [self performSelectorOnMainThread:@selector(updateTitleBar) withObject:nil waitUntilDone:NO];
     }
     else if (_id == DB_EV_VOLUMECHANGED) {
-        [g_appDelegate performSelectorOnMainThread:@selector(volumeChanged) withObject:nil waitUntilDone:NO];
+        [self performSelectorOnMainThread:@selector(volumeChanged) withObject:nil waitUntilDone:NO];
     }
     else if (_id == DB_EV_OUTPUTCHANGED) {
-        [g_appDelegate performSelectorOnMainThread:@selector(outputDeviceChanged) withObject:nil waitUntilDone:NO];
+        [self performSelectorOnMainThread:@selector(outputDeviceChanged) withObject:nil waitUntilDone:NO];
     }
+    else if (_id == DB_EV_TERMINATE) {
+        [NSNotificationCenter.defaultCenter postNotificationName:@"ApplicationWillQuit" object:nil];
+    }
+
+    [self.mainWindow message:_id ctx:ctx p1:p1 p2:p2];
 
     return 0;
 }
@@ -743,7 +795,7 @@ main_cleanup_and_quit (void);
 }
 
 - (void) updateDockNowPlaying {
-    DB_playItem_t *it = deadbeef->streamer_get_playing_track ();
+    DB_playItem_t *it = deadbeef->streamer_get_playing_track_safe ();
     if (!it) {
         [self clearDockNowPlaying];
         return;
@@ -783,9 +835,9 @@ main_cleanup_and_quit (void);
     if (it) {
         deadbeef->pl_item_unref (it);
     }
-    _dockMenuNPTitle = [[NSMenuItem alloc] initWithTitle:[NSString stringWithUTF8String:title] action:nil keyEquivalent:@""];
+    _dockMenuNPTitle = [[NSMenuItem alloc] initWithTitle:@(title) action:nil keyEquivalent:@""];
     _dockMenuNPTitle.enabled = NO;
-    _dockMenuNPArtistAlbum = [[NSMenuItem alloc] initWithTitle:[NSString stringWithUTF8String:artistAlbum] action:nil keyEquivalent:@""];
+    _dockMenuNPArtistAlbum = [[NSMenuItem alloc] initWithTitle:@(artistAlbum) action:nil keyEquivalent:@""];
     _dockMenuNPArtistAlbum.enabled = NO;
     [_dockMenu insertItem:_dockMenuNPTitle atIndex:1];
     [_dockMenu insertItem:_dockMenuNPArtistAlbum atIndex:2];
@@ -800,7 +852,7 @@ main_cleanup_and_quit (void);
 - (IBAction)newPlaylistAction:(id)sender {
     int playlist = cocoaui_add_new_playlist ();
     if (playlist != -1) {
-        cocoaui_playlist_set_curr (playlist);
+        deadbeef->plt_set_curr_idx (playlist);
     }
 }
 
@@ -811,23 +863,38 @@ main_cleanup_and_quit (void);
     openDlg.canChooseDirectories = NO;
     if ([openDlg runModal] == NSModalResponseOK)
     {
-        NSArray* files = [openDlg URLs];
-        if ([files count] < 1) {
+        NSArray* files = openDlg.URLs;
+        if (files.count < 1) {
             return;
         }
-        NSString *fname = [[files firstObject] path];
-        dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
-        dispatch_async(aQueue, ^{
-            ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-            if (!deadbeef->plt_add_files_begin (plt, 0)) {
-                    deadbeef->plt_clear (plt);
-                    int abort = 0;
-                    deadbeef->plt_load2 (0, plt, NULL, [fname UTF8String], &abort, NULL, NULL);
-                    deadbeef->plt_save_config (plt);
-                    deadbeef->plt_add_files_end (plt, 0);
-            }
-            deadbeef->plt_unref (plt);
+        NSString *fname = [files.firstObject path];
+        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
+            ddb_playlist_t *plt = deadbeef->plt_alloc ("load-playlist");
+            ddb_playlist_t *plt_curr = deadbeef->plt_get_curr ();
+            deadbeef->plt_clear (plt_curr);
             deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
+            if (!deadbeef->plt_add_files_begin (plt_curr, 0)) {
+                int abort = 0;
+                deadbeef->plt_load2 (0, plt, NULL, fname.UTF8String, &abort, NULL, NULL);
+                if (!abort) {
+                    dispatch_async(dispatch_get_main_queue(), ^{
+                        deadbeef->plt_move_all_items(plt_curr, plt, NULL);
+                        deadbeef->plt_save_config (plt);
+                        deadbeef->plt_add_files_end (plt_curr, 0);
+                        deadbeef->plt_unref (plt);
+                        deadbeef->plt_unref (plt_curr);
+                    });
+                }
+                else {
+                    deadbeef->plt_add_files_end (plt_curr, 0);
+                    deadbeef->plt_unref (plt);
+                    deadbeef->plt_unref (plt_curr);
+                }
+            }
+            else {
+                deadbeef->plt_unref (plt);
+                deadbeef->plt_unref (plt_curr);
+            }
         });
     }
 }
@@ -849,7 +916,7 @@ main_cleanup_and_quit (void);
             const char **exts = plug[i]->extensions;
             if (exts && plug[i]->save) {
                 for (int e = 0; exts[e]; e++) {
-                    NSString *ext = [NSString stringWithUTF8String:exts[e]];
+                    NSString *ext = @(exts[e]);
                     [types addObject:ext];
 
                     message = [message stringByAppendingString:@", "];
@@ -864,11 +931,11 @@ main_cleanup_and_quit (void);
     panel.allowsOtherFileTypes = NO;
 
     if ([panel runModal] == NSModalResponseOK) {
-        NSString *fname = [[panel URL] path];
+        NSString *fname = panel.URL.path;
         if (fname) {
             ddb_playlist_t *plt = deadbeef->plt_get_curr ();
             if (plt) {
-                deadbeef->plt_save (plt, NULL, NULL, [fname UTF8String], NULL, NULL, NULL);
+                deadbeef->plt_save (plt, NULL, NULL, fname.UTF8String, NULL, NULL, NULL);
                 deadbeef->plt_unref (plt);
             }
         }
@@ -898,10 +965,84 @@ main_cleanup_and_quit (void);
     if (!_helpWindow) {
         _helpWindow = [[HelpWindowController alloc] initWithWindowNibName:@"HelpViewer"];
     }
+    _helpWindow.contentURL = [NSBundle.mainBundle URLForResource:@"help-cocoa" withExtension:@"txt"];
 
-    if (![_helpWindow.window isVisible]) {
+    if (!(_helpWindow.window).visible) {
         [_helpWindow showWindow:self];
     }
 }
 
+- (IBAction)showChangelog:(id)sender {
+    if (!_helpWindow) {
+        _helpWindow = [[HelpWindowController alloc] initWithWindowNibName:@"HelpViewer"];
+    }
+    _helpWindow.contentURL = [NSBundle.mainBundle URLForResource:@"ChangeLog" withExtension:@""];
+
+    if (!(_helpWindow.window).visible) {
+        [_helpWindow showWindow:self];
+    }
+}
+
+- (IBAction)showGPL:(id)sender {
+    if (!_helpWindow) {
+        _helpWindow = [[HelpWindowController alloc] initWithWindowNibName:@"HelpViewer"];
+    }
+    _helpWindow.contentURL = [NSBundle.mainBundle URLForResource:@"COPYING" withExtension:@"GPLv2"];
+
+    if (!(_helpWindow.window).visible) {
+        [_helpWindow showWindow:self];
+    }
+}
+
+- (IBAction)showLGPL:(id)sender {
+    if (!_helpWindow) {
+        _helpWindow = [[HelpWindowController alloc] initWithWindowNibName:@"HelpViewer"];
+    }
+    _helpWindow.contentURL = [NSBundle.mainBundle URLForResource:@"COPYING.LGPLv2" withExtension:@"1"];
+
+    if (!(_helpWindow.window).visible) {
+        [_helpWindow showWindow:self];
+    }
+}
+
+
+- (IBAction)toggleDesignModeAction:(id)sender {
+    if (!self.designModeState.enabled && !deadbeef->conf_get_int("cocoaui.suppress_designmode_help", 0)) {
+        NSAlert *alert = [NSAlert new];
+        alert.messageText = @"Design Mode";
+        alert.informativeText = @"Use the right click menu to customize UI elements. Use Splitter and Tabs elements to place multiple elements.";
+        alert.showsSuppressionButton = YES;
+        alert.suppressionButton.title = @"Do not show this message again";
+        alert.alertStyle = NSAlertStyleInformational;
+
+        [alert beginSheetModalForWindow:self.mainWindow.window completionHandler:^(NSModalResponse returnCode) {
+            if (alert.suppressionButton.state == NSControlStateValueOn) {
+                deadbeef->conf_set_int ("cocoaui.suppress_designmode_help", 1);
+                deadbeef->conf_save ();
+            }
+        }];
+    }
+
+    self.designModeState.enabled = !self.designModeState.enabled;
+    self.designModeMenuItem.state = self.designModeState.enabled ? NSControlStateValueOn : NSControlStateValueOff;
+}
+
+- (IBAction)displayTrackProperties:(id)sender {
+    [TrackPropertiesManager.shared displayTrackProperties];
+}
+
+- (IBAction)editKeybooardShortcutsAction:(id)sender {
+    if (self.keyboardShortcutEditorWindowController == nil) {
+        self.keyboardShortcutEditorWindowController = [[KeyboardShortcutEditorWindowController alloc] initWithWindowNibName:@"KeyboardShortcutEditorWindowController"];
+        self.keyboardShortcutEditorViewController = [KeyboardShortcutEditorViewController new];
+        self.keyboardShortcutEditorViewController.model = KeyboardShortcutManager.shared;
+        self.keyboardShortcutEditorWindowController.window.contentView = self.keyboardShortcutEditorViewController.view;
+    }
+
+    KeyboardShortcutViewItem *viewItem = [KeyboardShortcutManager.shared createViewItems];
+    [self.keyboardShortcutEditorViewController updateWithViewItem:viewItem];
+
+    [self.keyboardShortcutEditorWindowController showWindow:nil];
+}
+
 @end
